@@ -50,6 +50,7 @@
 #include <hash.h>
 #include <integrity.h>
 #include <processor.h>
+#include <efi_memmap.h>
 
 extern loader_ctx *g_ldr_ctx;
 
@@ -65,11 +66,11 @@ printk_long(const char *what)
 {
     /* chunk the command line into 70 byte chunks */
 #define CHUNK_SIZE 70
-    int      cmdlen = strlen(what);
+    int      cmdlen = tb_strlen(what);
     const char    *cptr = what;
     char     cmdchunk[CHUNK_SIZE+1];
     while (cmdlen > 0) {
-        strncpy(cmdchunk, cptr, CHUNK_SIZE);
+        tb_strncpy(cmdchunk, cptr, CHUNK_SIZE);
         cmdchunk[CHUNK_SIZE] = 0;
         printk(TBOOT_INFO"\t%s\n", cmdchunk);
         cmdlen -= CHUNK_SIZE;
@@ -173,12 +174,14 @@ bool expand_linux_image(const void *linux_image, size_t linux_size,
         if ( mem_limit > 0x100000000ULL || mem_limit == 0 )
             mem_limit = 0x100000000ULL;
 
-        uint64_t max_ram_base, max_ram_size;
-        get_highest_sized_ram(initrd_size, mem_limit,
-                              &max_ram_base, &max_ram_size);
-        if ( max_ram_size == 0 ) {
-            printk(TBOOT_ERR"not enough RAM for initrd\n");
-            return false;
+        uint64_t max_ram_base = 0, max_ram_size = 0;
+        if (!efi_memmap_get_highest_sized_ram(initrd_size, mem_limit,
+                                              &max_ram_base, &max_ram_size)) {
+            if (!e820_get_highest_sized_ram(initrd_size, mem_limit,
+                                            &max_ram_base, &max_ram_size)) {
+                printk(TBOOT_ERR"not enough RAM for initrd\n");
+                return false;
+            }
         }
         if ( initrd_size > max_ram_size ) {
             printk(TBOOT_ERR"initrd_size is too large\n");
@@ -218,7 +221,7 @@ bool expand_linux_image(const void *linux_image, size_t linux_size,
             }
         }
 
-        memmove((void *)initrd_base, initrd_image, initrd_size);
+        tb_memmove((void *)initrd_base, initrd_image, initrd_size);
         printk(TBOOT_DETA"Initrd from 0x%lx to 0x%lx\n",
                (unsigned long)initrd_base,
                (unsigned long)(initrd_base + initrd_size));
@@ -305,32 +308,36 @@ bool expand_linux_image(const void *linux_image, size_t linux_size,
     /* set cmd_line_ptr */
     hdr->cmd_line_ptr = real_mode_base + KERNEL_CMDLINE_OFFSET;
 
-    /* Save linux header struct to temp memory, in case the it is overwritten by memmove below*/
+    /* Save linux header struct to temp memory, in case the it is overwritten by tb_memmove below*/
     linux_kernel_header_t temp_hdr;
-    memmove(&temp_hdr, hdr, sizeof(temp_hdr));
+    tb_memmove(&temp_hdr, hdr, sizeof(temp_hdr));
     hdr = &temp_hdr;
 
     /* load protected-mode part */
-    memmove((void *)protected_mode_base, linux_image + real_mode_size,
+    tb_memmove((void *)protected_mode_base, linux_image + real_mode_size,
             protected_mode_size);
     printk(TBOOT_DETA"Kernel (protected mode) from 0x%lx to 0x%lx\n",
            (unsigned long)protected_mode_base,
            (unsigned long)(protected_mode_base + protected_mode_size));
 
     /* load real-mode part */
-    memmove((void *)real_mode_base, linux_image, real_mode_size);
+    tb_memmove((void *)real_mode_base, linux_image, real_mode_size);
     printk(TBOOT_DETA"Kernel (real mode) from 0x%lx to 0x%lx\n",
            (unsigned long)real_mode_base,
            (unsigned long)(real_mode_base + real_mode_size));
 
     /* copy cmdline */
     const char *kernel_cmdline = get_cmdline(g_ldr_ctx);
+    if ( kernel_cmdline == NULL ) {
+        printk(TBOOT_ERR"Error: kernel cmdline not available\n");
+        return false;
+    }
     const size_t kernel_cmdline_size = REAL_END_OFFSET - KERNEL_CMDLINE_OFFSET;
-    size_t kernel_cmdline_strlen = strlen(kernel_cmdline);
+    size_t kernel_cmdline_strlen = tb_strlen(kernel_cmdline);
     if (kernel_cmdline_strlen > kernel_cmdline_size - 1)
         kernel_cmdline_strlen = kernel_cmdline_size - 1;
-    memset((void *)hdr->cmd_line_ptr, 0, kernel_cmdline_size);
-    memcpy((void *)hdr->cmd_line_ptr, kernel_cmdline, kernel_cmdline_strlen);
+    tb_memset((void *)hdr->cmd_line_ptr, 0, kernel_cmdline_size);
+    tb_memcpy((void *)hdr->cmd_line_ptr, kernel_cmdline, kernel_cmdline_strlen);
 
     printk(TBOOT_INFO"Linux cmdline from 0x%lx to 0x%lx:\n",
            (unsigned long)hdr->cmd_line_ptr,
@@ -339,14 +346,12 @@ bool expand_linux_image(const void *linux_image, size_t linux_size,
 
     /* need to put boot_params in real mode area so it gets mapped */
     boot_params = (boot_params_t *)(real_mode_base + real_mode_size);
-    memset(boot_params, 0, sizeof(*boot_params));
-    memcpy(&boot_params->hdr, hdr, sizeof(*hdr));
+    tb_memset(boot_params, 0, sizeof(*boot_params));
+    tb_memcpy(&boot_params->hdr, hdr, sizeof(*hdr));
 
     /* need to handle a few EFI things here if such is our parentage */
     if (is_loader_launch_efi(g_ldr_ctx)){
         struct efi_info *efi = (struct efi_info *)(boot_params->efi_info);
-        struct screen_info_t *scr = 
-            (struct screen_info_t *)(boot_params->screen_info);
         uint32_t address = 0;
         uint64_t long_address = 0UL;
 
@@ -355,7 +360,7 @@ bool expand_linux_image(const void *linux_image, size_t linux_size,
 
 
         /* loader signature */
-        memcpy(&efi->efi_ldr_sig, "EL64", sizeof(uint32_t));
+        tb_memcpy(&efi->efi_ldr_sig, "EL64", sizeof(uint32_t));
 
         /* EFI system table addr */
         {
@@ -372,8 +377,8 @@ bool expand_linux_image(const void *linux_image, size_t linux_size,
             }
         }
 
-        efi_mmap_addr = find_efi_memmap(g_ldr_ctx, &descr_size,
-                                        &descr_vers, &mmap_size);
+        efi_mmap_addr = efi_memmap_get_addr(&descr_size, &descr_vers,
+                                            &mmap_size);
         if (!efi_mmap_addr) {
             printk(TBOOT_INFO"failed to get EFI memory map\n");
             efi->efi_memdescr_size = 0x1; // Avoid div by 0 in kernel.
@@ -390,14 +395,29 @@ bool expand_linux_image(const void *linux_image, size_t linux_size,
              */
             efi->efi_memmap_hi = 0;
 
+            if (get_tboot_dump_memmap()) {
+                printk(TBOOT_INFO"EFI memory map after modifications:\n");
+                efi_memmap_dump();
+            }
+
             printk(TBOOT_INFO "EFI memmap: memmap base: 0x%x, memmap size: 0x%x\n",
                   efi->efi_memmap, efi->efi_memmap_size);
             printk(TBOOT_INFO "EFI memmap: descr size: 0x%x, descr version: 0x%x\n",
                   efi->efi_memdescr_size, efi->efi_memdescr_ver);
          }
 
-        /* if we're here, GRUB2 probably threw a framebuffer tag at us */
-        load_framebuffer_info(g_ldr_ctx, (void *)scr);
+    }
+
+    screen_info_t *scr = (screen_info_t *)(boot_params->screen_info);
+    if (!load_framebuffer_info(g_ldr_ctx, (void *)scr,
+                               is_loader_launch_efi(g_ldr_ctx))) {
+        /* Fallback to 80x25 mode */
+        scr->orig_video_mode = 3;       /* BIOS 80*25 text mode */
+        scr->orig_video_lines = 25;
+        scr->orig_video_cols = 80;
+        scr->orig_video_points = 16;    /* set font height to 16 pixels */
+        scr->orig_video_isVGA = 1;      /* use VGA text screen setups */
+        scr->orig_y = 24;               /* start display text @ screen end*/
     }
     
     /* detect e820 table */
@@ -405,6 +425,10 @@ bool expand_linux_image(const void *linux_image, size_t linux_size,
         int i;
 
         memory_map_t *p = get_loader_memmap(g_ldr_ctx);
+        if ( p == NULL ) {
+            printk(TBOOT_ERR"Error: no memory map available\n");
+            return false;
+        }
         uint32_t memmap_start = (uint32_t) p;
         uint32_t memmap_length = get_loader_memmap_length(g_ldr_ctx);
         for ( i = 0; (uint32_t)p < memmap_start + memmap_length; i++ )
@@ -419,15 +443,15 @@ bool expand_linux_image(const void *linux_image, size_t linux_size,
         boot_params->e820_entries = i;
     }
 
-    if (0 == is_loader_launch_efi(g_ldr_ctx)){
-        screen_info_t *screen = (screen_info_t *)&boot_params->screen_info;
-        screen->orig_video_mode = 3;       /* BIOS 80*25 text mode */
-        screen->orig_video_lines = 25;
-        screen->orig_video_cols = 80;
-        screen->orig_video_points = 16;    /* set font height to 16 pixels */
-        screen->orig_video_isVGA = 1;      /* use VGA text screen setups */
-        screen->orig_y = 24;               /* start display text @ screen end*/
-    }
+    // if (0 == is_loader_launch_efi(g_ldr_ctx)){
+    //     screen_info_t *screen = (screen_info_t *)&boot_params->screen_info;
+    //     screen->orig_video_mode = 3;       /* BIOS 80*25 text mode */
+    //     screen->orig_video_lines = 25;
+    //     screen->orig_video_cols = 80;
+    //     screen->orig_video_points = 16;    /* set font height to 16 pixels */
+    //     screen->orig_video_isVGA = 1;      /* use VGA text screen setups */
+    //     screen->orig_y = 24;               /* start display text @ screen end*/
+    // }
 
     /* set address of tboot shared page */
     if ( is_measured_launch )
