@@ -70,6 +70,8 @@
 #include <integrity.h>
 #include <cmdline.h>
 #include <tpm_20.h>
+#include <vtd.h>
+#include <efi_memmap.h>
 
 extern void _prot_to_real(uint32_t dist_addr);
 extern bool set_policy(void);
@@ -137,25 +139,42 @@ static void copy_s3_wakeup_entry(void)
     }
 
     /* backup target address space first */
-    memcpy(g_saved_s3_wakeup_page, (void *)TBOOT_S3_WAKEUP_ADDR,
+    tb_memcpy(g_saved_s3_wakeup_page, (void *)TBOOT_S3_WAKEUP_ADDR,
            s3_wakeup_end - s3_wakeup_16);
 
     /* copy s3 entry into target mem */
-    memcpy((void *)TBOOT_S3_WAKEUP_ADDR, s3_wakeup_16,
+    tb_memcpy((void *)TBOOT_S3_WAKEUP_ADDR, s3_wakeup_16,
            s3_wakeup_end - s3_wakeup_16);
 }
 
 static void restore_saved_s3_wakeup_page(void)
 {
     /* restore saved page */
-    memcpy((void *)TBOOT_S3_WAKEUP_ADDR, g_saved_s3_wakeup_page,
+    tb_memcpy((void *)TBOOT_S3_WAKEUP_ADDR, g_saved_s3_wakeup_page,
            s3_wakeup_end - s3_wakeup_16);
 }
 
+static inline void print_tboot_shared(const tboot_shared_t *tboot_shared)
+{
+    printk(TBOOT_DETA"tboot_shared data:\n");
+    printk(TBOOT_DETA"\t version: %d\n", tboot_shared->version);
+    printk(TBOOT_DETA"\t log_addr: 0x%08x\n", tboot_shared->log_addr);
+    printk(TBOOT_DETA"\t shutdown_entry: 0x%08x\n", tboot_shared->shutdown_entry);
+    printk(TBOOT_DETA"\t shutdown_type: %d\n", tboot_shared->shutdown_type);
+    printk(TBOOT_DETA"\t tboot_base: 0x%08x\n", tboot_shared->tboot_base);
+    printk(TBOOT_DETA"\t tboot_size: 0x%x\n", tboot_shared->tboot_size);
+    printk(TBOOT_DETA"\t num_in_wfs: %u\n", tboot_shared->num_in_wfs);
+    printk(TBOOT_DETA"\t flags: 0x%8.8x\n", tboot_shared->flags);
+    printk(TBOOT_DETA"\t ap_wake_addr: 0x%08x\n", (uint32_t)tboot_shared->ap_wake_addr);
+    printk(TBOOT_DETA"\t ap_wake_trigger: %u\n", tboot_shared->ap_wake_trigger);
+}
+
 static void post_launch(void)
 {
     uint64_t base, size;
     tb_error_t err;
+    struct tpm_if *tpm = get_tpm();
+    const struct tpm_if_fp *tpm_fp = get_tpm_fp();
     extern tboot_log_t *g_log;
     extern void shutdown_entry(void);
 
@@ -167,7 +186,8 @@ static void post_launch(void)
     txt_post_launch();
 
     /* backup DMAR table */
-    save_vtd_dmar_table();
+    if ( get_tboot_save_vtd() )
+        vtd_save_dmar_table();
 
     if ( s3_flag  )    
          s3_launch();
@@ -204,22 +224,10 @@ static void post_launch(void)
     printk(TBOOT_INFO"protecting tboot (%Lx - %Lx) in e820 table\n", base,      (base + size - 1));
     if ( !e820_protect_region(base, size, mem_type) )      
         apply_policy(TB_ERR_FATAL);
-
-    /* if using memory logging, reserve log area */
-    if ( g_log_targets & TBOOT_LOG_TARGET_MEMORY ) {
-        base = TBOOT_SERIAL_LOG_ADDR;
-        size = TBOOT_SERIAL_LOG_SIZE;
-        printk(TBOOT_INFO"reserving tboot memory log (%Lx - %Lx) in e820 table\n", base, (base + size - 1));
-        if ( !e820_protect_region(base, size, E820_RESERVED) )         
-            apply_policy(TB_ERR_FATAL);
+    if (!efi_memmap_reserve(base, size)) {
+        apply_policy(TB_ERR_FATAL);
     }
 
-    /* replace map in loader context with copy */
-    replace_e820_map(g_ldr_ctx);
-
-    printk(TBOOT_DETA"adjusted e820 map:\n");
-    print_e820_map();
-
     /*
      * verify modules against policy
      */
@@ -228,7 +236,7 @@ static void post_launch(void)
     /*
      * verify nv indices against policy
      */
-    if ( (g_tpm->major == TPM12_VER_MAJOR) &&  get_tboot_measure_nv() ) 
+    if ( (tpm->major == TPM12_VER_MAJOR) &&  get_tboot_measure_nv() ) 
 	verify_all_nvindices();
 
     /*
@@ -238,14 +246,14 @@ static void post_launch(void)
 	apply_policy(TB_ERR_S3_INTEGRITY);
 
 	
-    if ( g_tpm->major == TPM20_VER_MAJOR ) {
-	g_tpm->context_save(g_tpm, g_tpm->cur_loc, handle2048, &tpm2_context_saved);
+    if ( tpm->major == TPM20_VER_MAJOR ) {
+	tpm_fp->context_save(tpm, tpm->cur_loc, handle2048, &tpm2_context_saved);
     }
 
 	/*
      * init MLE/kernel shared data page
      */
-    memset(&_tboot_shared, 0, PAGE_SIZE);
+    tb_memset(&_tboot_shared, 0, PAGE_SIZE);
     _tboot_shared.uuid = (uuid_t)TBOOT_SHARED_UUID;
     _tboot_shared.version = 6;
     _tboot_shared.log_addr = (uint32_t)g_log;
@@ -253,7 +261,7 @@ static void post_launch(void)
     _tboot_shared.tboot_base = (uint32_t)&_start;
     _tboot_shared.tboot_size = (uint32_t)&_end - (uint32_t)&_start;
     uint32_t key_size = sizeof(_tboot_shared.s3_key);
-    if ( !g_tpm->get_random(g_tpm, 2, _tboot_shared.s3_key, &key_size) || key_size != sizeof(_tboot_shared.s3_key) )
+    if ( !tpm_fp->get_random(tpm, 2, _tboot_shared.s3_key, &key_size) || key_size != sizeof(_tboot_shared.s3_key) )
         apply_policy(TB_ERR_S3_INTEGRITY);
     _tboot_shared.num_in_wfs = atomic_read(&ap_wfs_count);
     if ( use_mwait() ) {
@@ -348,20 +356,29 @@ void begin_launch(void *addr, uint32_t magic)
            // cmdline = skip_filename(cmdline_orig);
             cmdline = cmdline_orig;
         }
-        memset(g_cmdline, '\0', sizeof(g_cmdline));
+        tb_memset(g_cmdline, '\0', sizeof(g_cmdline));
         if (cmdline)
-            strncpy(g_cmdline, cmdline, sizeof(g_cmdline)-1);
+            tb_strncpy(g_cmdline, cmdline, sizeof(g_cmdline)-1);
     }
 
     /* always parse cmdline */
     tboot_parse_cmdline();
 
     /* initialize all logging targets */
-    printk_init();
+    if ((is_launched() && get_framebuffer_info(g_ldr_ctx) != NULL) || s3_flag) {
+        /* 
+         * We are using framebuffer method for printing VGA logs. It does not
+         * when PMRs are programmed, so in post-launch we have to disable VGA
+         * logging.
+         */
+        printk_init(true);
+    } else {
+        printk_init(false);
+    }
 
-    printk(TBOOT_INFO"******************* TBOOT *******************\n");
+    printk(TBOOT_INFO"*********************** TBOOT ***********************\n");
     printk(TBOOT_INFO"   %s\n", TBOOT_CHANGESET);
-    printk(TBOOT_INFO"*********************************************\n");
+    printk(TBOOT_INFO"*****************************************************\n");
 
     printk(TBOOT_INFO"command line: %s\n", g_cmdline);
     /* if telled to check revocation acm result, go with simplified path */
@@ -389,6 +406,12 @@ void begin_launch(void *addr, uint32_t magic)
     /* make copy of e820 map that we will use and adjust */
     if ( !s3_flag ) {
         if ( !copy_e820_map(g_ldr_ctx) )  apply_policy(TB_ERR_FATAL);
+        if (efi_memmap_copy(g_ldr_ctx)) {
+            if (get_tboot_dump_memmap()) {
+                printk(TBOOT_INFO"Original EFI memory map:\n");
+                efi_memmap_dump();
+            }
+        }
     }
 
     /* we need to make sure this is a (TXT-) capable platform before using */
@@ -475,18 +498,19 @@ void begin_launch(void *addr, uint32_t magic)
 
 void s3_launch(void)
 {
+    struct tpm_if *tpm = get_tpm();
+    const struct tpm_if_fp *tpm_fp = get_tpm_fp();
     /* restore backed-up s3 wakeup page */
     restore_saved_s3_wakeup_page();
-
-	/* load saved tpm2 context for unseal */
-	if ( g_tpm->major == TPM20_VER_MAJOR ) {
-	    g_tpm->context_flush(g_tpm, g_tpm->cur_loc, handle2048);
-	    g_tpm->context_load(g_tpm, g_tpm->cur_loc, &tpm2_context_saved, &handle2048);
-	}
-
+    /* load saved tpm2 context for unseal */
+    if ( tpm->major == TPM20_VER_MAJOR ) {
+        tpm_fp->context_flush(tpm, tpm->cur_loc, handle2048);
+        tpm_fp->context_load(tpm, tpm->cur_loc, &tpm2_context_saved, &handle2048);
+    }
 
     /* remove DMAR table if necessary */
-    remove_vtd_dmar_table();
+    if ( get_tboot_save_vtd() )
+        vtd_remove_dmar_table();
 
     if ( !is_launched() )
         apply_policy(TB_ERR_S3_INTEGRITY);
@@ -498,6 +522,15 @@ void s3_launch(void)
     }
 
     print_tboot_shared(&_tboot_shared);
+    
+    if (g_tpm_family != TPM_IF_20_CRB ) {
+        if (!release_locality(tpm->cur_loc))
+            printk(TBOOT_ERR"Release TPM FIFO locality %d failed \n", tpm->cur_loc);
+    }
+    else {
+        if (!tpm_relinquish_locality_crb(tpm->cur_loc))
+            printk(TBOOT_ERR"Relinquish TPM CRB locality %d failed \n", tpm->cur_loc);
+    }
 
     /* (optionally) pause when transferring kernel resume */
     if ( g_vga_delay > 0 )
@@ -514,9 +547,11 @@ static void shutdown_system(uint32_t shutdown_type)
     char type[32];
 
     if ( shutdown_type >= ARRAY_SIZE(types) )
-        snprintf(type, sizeof(type), "unknown: %u", shutdown_type);
-    else
-        strncpy(type, types[shutdown_type], sizeof(type));
+        tb_snprintf(type, sizeof(type), "unknown: %u", shutdown_type);
+    else {
+        tb_strncpy(type, types[shutdown_type], sizeof(type));
+        type[sizeof(type) - 1] = '\0';
+    }
     printk(TBOOT_INFO"shutdown_system() called for shutdown_type: %s\n", type);
 
     switch( shutdown_type ) {
@@ -560,6 +595,9 @@ static void shutdown_system(uint32_t shutdown_type)
 
 void shutdown(void)
 {
+    struct tpm_if *tpm = get_tpm();
+    const struct tpm_if_fp *tpm_fp = get_tpm_fp();
+   
     /* wait-for-sipi only invoked for APs, so skip all BSP shutdown code */
     if ( _tboot_shared.shutdown_type == TB_SHUTDOWN_WFS ) {
         atomic_inc(&ap_wfs_count);
@@ -585,24 +623,25 @@ void shutdown(void)
             printk(TBOOT_ERR"Release TPM FIFO locality 0 failed \n");
         if (!release_locality(1))
             printk(TBOOT_ERR"Release TPM FIFO locality 1 failed \n");
-        if (!tpm_wait_cmd_ready(g_tpm->cur_loc))
-            printk(TBOOT_ERR"Request TPM FIFO locality %d failed \n", g_tpm->cur_loc);
+        if (!tpm_wait_cmd_ready(tpm->cur_loc))
+            printk(TBOOT_ERR"Request TPM FIFO locality %d failed \n", tpm->cur_loc);
     }
     else {
         if (!tpm_relinquish_locality_crb(0))
             printk(TBOOT_ERR"Release TPM CRB locality 0 failed \n");
         if (!tpm_relinquish_locality_crb(1))			 
             printk(TBOOT_ERR"Release TPM CRB locality 1 failed \n");
-        if (!tpm_request_locality_crb(g_tpm->cur_loc))
-            printk(TBOOT_ERR"Request TPM CRB locality %d failed \n", g_tpm->cur_loc);
+        if (!tpm_request_locality_crb(tpm->cur_loc))
+            printk(TBOOT_ERR"Request TPM CRB locality %d failed \n", tpm->cur_loc);
     }
 
     if ( _tboot_shared.shutdown_type == TB_SHUTDOWN_S3 ) {
         /* restore DMAR table if needed */
-        restore_vtd_dmar_table();
-	if ( g_tpm->major == TPM20_VER_MAJOR ) {
-	    g_tpm->context_flush(g_tpm, g_tpm->cur_loc, handle2048);
-	    g_tpm->context_load(g_tpm, g_tpm->cur_loc, &tpm2_context_saved, &handle2048);
+        if ( get_tboot_save_vtd() )
+            vtd_restore_dmar_table();
+	if ( tpm->major == TPM20_VER_MAJOR ) {
+	    tpm_fp->context_flush(tpm, tpm->cur_loc, handle2048);
+	    tpm_fp->context_load(tpm, tpm->cur_loc, &tpm2_context_saved, &handle2048);
  	}
 
 		
@@ -617,20 +656,26 @@ void shutdown(void)
                error */
         else
             /* wipe S3 key from memory now that it is sealed */
-            memset(_tboot_shared.s3_key, 0, sizeof(_tboot_shared.s3_key));
+            tb_memset(_tboot_shared.s3_key, 0, sizeof(_tboot_shared.s3_key));
     }
 
     /* cap dynamic PCRs extended as part of launch (17, 18, ...) */
     if ( is_launched() ) {
 
         /* cap PCRs to ensure no follow-on code can access sealed data */
-        g_tpm->cap_pcrs(g_tpm, g_tpm->cur_loc, -1);
+        if (!tpm_fp->cap_pcrs(tpm, tpm->cur_loc, -1)) {
+            printk(TBOOT_ERR"failed to cap PCRs\n");
+            apply_policy(TB_ERR_FATAL);
+        }
 
         /* have TPM save static PCRs (in case VMM/kernel didn't) */
         /* per TCG spec, TPM can invalidate saved state if any other TPM
            operation is performed afterwards--so do this last */
-        if ( _tboot_shared.shutdown_type == TB_SHUTDOWN_S3 )
-            g_tpm->save_state(g_tpm, g_tpm->cur_loc);
+        if ( _tboot_shared.shutdown_type == TB_SHUTDOWN_S3 ) {
+            if (tpm_fp->save_state(tpm, tpm->cur_loc) != 0) {
+                printk(TBOOT_ERR"failed to save TPM state\n");
+            }
+        }
 
         /* scrub any secrets by clearing their memory, then flush cache */
         /* we don't have any secrets to scrub, however */
