@@ -53,6 +53,8 @@
 #include <hash.h>
 #include <integrity.h>
 #include <cmdline.h>
+#include <vtd.h>
+#include <efi_memmap.h>
 #include <txt/txt.h>
 #include <txt/smx.h>
 #include <txt/mtrrs.h>
@@ -254,6 +256,9 @@ static bool reserve_vtd_delta_mem(uint64_t min_lo_ram, uint64_t max_lo_ram,
                base, base + length);
         if ( !e820_reserve_ram(base, length) )
             return false;
+        if (!efi_memmap_reserve(base, length)) {
+           return false;
+        }
     }
     if ( max_hi_ram != (os_sinit_data->vtd_pmr_hi_base +
                         os_sinit_data->vtd_pmr_hi_size) ) {
@@ -263,6 +268,9 @@ static bool reserve_vtd_delta_mem(uint64_t min_lo_ram, uint64_t max_lo_ram,
                base, base + length);
         if ( !e820_reserve_ram(base, length) )
             return false;
+        if (!efi_memmap_reserve(base, length)) {
+           return false;
+        }
     }
 
     return true;
@@ -305,7 +313,7 @@ static bool verify_vtd_pmrs(txt_heap_t *txt_heap)
     }
 
     /* compare to current values */
-    memset(&tmp_os_sinit_data, 0, sizeof(tmp_os_sinit_data));
+    tb_memset(&tmp_os_sinit_data, 0, sizeof(tmp_os_sinit_data));
     tmp_os_sinit_data.version = os_sinit_data->version;
     set_vtd_pmrs(&tmp_os_sinit_data, min_lo_ram, max_lo_ram, min_hi_ram,
                  max_hi_ram);
@@ -318,8 +326,8 @@ static bool verify_vtd_pmrs(txt_heap_t *txt_heap)
          (tmp_os_sinit_data.vtd_pmr_hi_size !=
           os_sinit_data->vtd_pmr_hi_size) ) {
         printk(TBOOT_ERR"OS to SINIT data VT-d PMR settings do not match:\n");
-        print_os_sinit_data(&tmp_os_sinit_data);
-        print_os_sinit_data(os_sinit_data);
+        print_os_sinit_data_vtdpmr(&tmp_os_sinit_data);
+        print_os_sinit_data_vtdpmr(os_sinit_data);
         return false;
     }
 
@@ -434,19 +442,19 @@ bool verify_e820_map(sinit_mdr_t* mdrs_base, uint32_t num_mdrs)
 
     /* sort mdrs */
     for( i = 0; i < num_mdrs; i++ ) {
-        memcpy(&tmp_entry, &mdrs_base[i], sizeof(sinit_mdr_t));
+        tb_memcpy(&tmp_entry, &mdrs_base[i], sizeof(sinit_mdr_t));
         pos = i;
         for ( j = i + 1; j < num_mdrs; j++ ) {
             if ( ( tmp_entry.base > mdrs_base[j].base )
                  || (( tmp_entry.base == mdrs_base[j].base ) &&
                      ( tmp_entry.length > mdrs_base[j].length )) ) {
-                memcpy(&tmp_entry, &mdrs_base[j], sizeof(sinit_mdr_t));
+                tb_memcpy(&tmp_entry, &mdrs_base[j], sizeof(sinit_mdr_t));
                 pos = j;
             }
         }
         if ( pos > i ) {
-            memcpy(&mdrs_base[pos], &mdrs_base[i], sizeof(sinit_mdr_t));
-            memcpy(&mdrs_base[i], &tmp_entry, sizeof(sinit_mdr_t));
+            tb_memcpy(&mdrs_base[pos], &mdrs_base[i], sizeof(sinit_mdr_t));
+            tb_memcpy(&mdrs_base[i], &tmp_entry, sizeof(sinit_mdr_t));
         }
     }
 
